package models

import (
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"
)

//type BaseFromBaiinfoData struct {
//	BaiinfoDataId          int `orm:"column(baiinfo_data_id);pk"`
//	BaseFromBaiinfoIndexId int
//	IndexCode              string
//	DataTime               string
//	Value                  string
//	CreateTime             time.Time
//	ModifyTime             time.Time
//	DataTimestamp          int64
//}

type BaseFromBaiinfoData struct {
	BaiinfoDataId          int       `gorm:"column:baiinfo_data_id;primaryKey"`
	BaseFromBaiinfoIndexId int       `gorm:"column:base_from_baiinfo_index_id"`
	IndexCode              string    `gorm:"column:index_code"`
	DataTime               string    `gorm:"column:data_time"`
	Value                  string    `gorm:"column:value"`
	CreateTime             time.Time `gorm:"column:create_time"`
	ModifyTime             time.Time `gorm:"column:modify_time"`
	DataTimestamp          int64     `gorm:"column:data_timestamp"`
}

func (v BaseFromBaiinfoData) TableName() string {
	return "base_from_baiinfo_data"
}

func GetBaseFromBaiinfoDataByCondition(condition string, pars []interface{}) (list []*BaseFromBaiinfoData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_baiinfo_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	//_, err = o.Raw(sql, pars).QueryRows(&list)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Find(&list).Error
	return
}

// AddEdbDataFromBaiinfo 新增百川盈弗指标数据
func AddEdbDataFromBaiinfo(edbCode string) (err error) {
	//o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	baiinfoBaseDataAll, err := GetBaseFromBaiinfoDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_baiinfo(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range baiinfoBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DmSQL.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

// RefreshEdbDataFromBaiinfo 刷新百川盈弗指标数据
func RefreshEdbDataFromBaiinfo(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_BAIINFO
	subSource := utils.DATA_SUB_SOURCE_EDB
	//o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	baiinfoDataList, err := GetBaseFromBaiinfoDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_baiinfo(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range baiinfoDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DmSQL.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// HandleBaiinfoExcelDataReq 百川盈弗的excel数据
type HandleBaiinfoExcelDataReq struct {
	BaseFilePath   string
	RenameFilePath string
	IndexName      string
	IndexCode      string
	ExcelDataMap   map[string]string
	TerminalCode   string `description:"编码"`
}

//type BaseFromBaiinfoIndex struct {
//	BaseFromBaiinfoIndexId int64 `orm:"column(base_from_baiinfo_index_id);pk"`
//	IndexCode              string
//	IndexName              string
//	Frequency              string
//	Unit                   string
//	StartDate              string
//	EndDate                string
//	Sort                   int
//	BaseFileName           string
//	RenameFileName         string
//	TerminalCode           string
//	CreateTime             time.Time
//	ModifyTime             time.Time
//}

type BaseFromBaiinfoIndex struct {
	BaseFromBaiinfoIndexId int64     `gorm:"column:base_from_baiinfo_index_id;primaryKey"`
	IndexCode              string    `gorm:"column:index_code"`
	IndexName              string    `gorm:"column:index_name"`
	Frequency              string    `gorm:"column:frequency"`
	Unit                   string    `gorm:"column:unit"`
	StartDate              string    `gorm:"column:start_date"`
	EndDate                string    `gorm:"column:end_date"`
	Sort                   int       `gorm:"column:sort"`
	BaseFileName           string    `gorm:"column:base_file_name"`
	RenameFileName         string    `gorm:"column:rename_file_name"`
	TerminalCode           string    `gorm:"column:terminal_code"`
	CreateTime             time.Time `gorm:"column:create_time"`
	ModifyTime             time.Time `gorm:"column:modify_time"`
}

func (d *BaseFromBaiinfoIndex) AddBaseFromBaiinfoIndex() (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(d)
	err = global.DEFAULT_DmSQL.Create(d).Error
	lastId = d.BaseFromBaiinfoIndexId
	return
}

func AddBaseFromBaiinfoData(item []*BaseFromBaiinfoData) (err error) {
	//o := orm.NewOrm()
	//_, err = o.InsertMulti(1, item)
	err = global.DEFAULT_DmSQL.CreateInBatches(item, 500).Error
	return
}

func (d *BaseFromBaiinfoIndex) GetSmmIndexItem(indexCode string) (item *BaseFromBaiinfoIndex, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_baiinfo_index WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DmSQL.Raw(sql, indexCode).First(&item).Error
	return
}

func GetBaseFromBaiinfoDataByCode(indexCode string) (list []*BaseFromBaiinfoData, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_baiinfo_data WHERE index_code=? `
	//_, err = o.Raw(sql, indexCode).QueryRows(&list)
	err = global.DEFAULT_DmSQL.Raw(sql, indexCode).Find(&list).Error
	return
}

func ModifyBaiinfoIndexSort(baseFromBaiinfoIndexId int64) (err error) {
	//o := orm.NewOrm()
	sql := ` UPDATE base_from_baiinfo_index SET sort=? WHERE base_from_baiinfo_index_id=? `
	//_, err = o.Raw(sql, baseFromBaiinfoIndexId, baseFromBaiinfoIndexId).Exec()
	err = global.DEFAULT_DmSQL.Exec(sql, baseFromBaiinfoIndexId, baseFromBaiinfoIndexId).Error
	return
}

func GetBaiinfoIndexInfoMaxAndMinInfo(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_baiinfo_data WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DmSQL.Raw(sql, indexCode).First(&item).Error
	return
}

func ModifyBaiinfoIndexMaxAndMinInfo(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	//o := orm.NewOrm()
	sql := ` UPDATE base_from_baiinfo_index SET start_date=?,end_date=?,modify_time=NOW() WHERE index_code=? `
	//_, err = o.Raw(sql, item.MinDate, item.MaxDate, indexCode).Exec()
	err = global.DEFAULT_DmSQL.Exec(sql, item.MinDate, item.MaxDate, indexCode).Error
	return
}

// Update 修改
func (r *BaseFromBaiinfoIndex) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(r, updateCols...)
	err = global.DEFAULT_DmSQL.Model(r).Updates(r).Error
	return
}
