package future_good

import (
	"eta_gn/eta_index_lib/global"
	"fmt"
	"strings"
	"time"
)

// ChartInfoFutureGoodProfit 商品利润图表-扩展信息
type ChartInfoFutureGoodProfit struct {
	ChartInfoId  int       `gorm:"primaryKey;autoIncrement;column:chart_info_id" description:"商品利润图表ID(chart_info表source=5的)"`
	ProfitName   string    `gorm:"column:profit_name" description:"利润名称"`
	ProfitNameEn string    `gorm:"column:profit_name_en" description:"利润英文名称"`
	XValue       string    `gorm:"column:x_value" description:"X轴数据值"`
	YValue       string    `gorm:"column:y_value" description:"Y轴数据值"`
	CreateTime   time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime   time.Time `gorm:"column:modify_time" description:"更新时间"`
}

type CorrelationInfo struct {
	LeadValue       int    `description:"领先值"`
	LeadUnit        string `description:"领先单位"`
	CalculateValue  int    `description:"计算窗口"`
	CalculateUnit   string `description:"计算频度"`
	StartDate       string `description:"开始日期"`
	EndDate         string `description:"结束日期"`
	EdbInfoIdFirst  int    `description:"A指标ID"`
	EdbInfoIdSecond int    `description:"B指标ID"`
	PeriodData      string `description:"X轴-期数数据"`
	CorrelationData string `description:"Y轴-商品利润系数"`
}

func (m *ChartInfoFutureGoodProfit) TableName() string {
	return "chart_info_future_good_profit"
}

func (m *ChartInfoFutureGoodProfit) Create() (err error) {
	err = global.DEFAULT_DmSQL.Create(m).Error
	if err != nil {
		return
	}
	//m.CorrelationChartInfoId = int(id)
	return
}

func (m *ChartInfoFutureGoodProfit) Update(cols []string) (err error) {

	err = global.DEFAULT_DmSQL.Model(m).Select(cols).Updates(m).Error
	return
}

func (m *ChartInfoFutureGoodProfit) Delete() (err error) {

	sql := fmt.Sprintf(`DELETE FROM %s WHERE chart_info_id = ? LIMIT 1`, m.TableName())
	err = global.DEFAULT_DmSQL.Exec(sql, m.ChartInfoId).Error
	return
}

func (m *ChartInfoFutureGoodProfit) GetItemById(id int) (err error) {

	sql := fmt.Sprintf(`SELECT * FROM %s WHERE chart_info_id = ? LIMIT 1`, m.TableName())
	err = global.DEFAULT_DmSQL.Raw(sql, id).First(&m).Error
	return
}

func (m *ChartInfoFutureGoodProfit) GetItemByCondition(condition string, pars []interface{}) (err error) {

	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s LIMIT 1`, m.TableName(), condition)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).First(&m).Error
	return
}

func (m *ChartInfoFutureGoodProfit) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*ChartInfoFutureGoodProfit, err error) {

	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Scan(&items).Error
	return
}
